@@ -11,44 +11,51 @@ impl Number {
 
 		if self.base2_exponent < 0 {
 			n = n.checked_shr(-self.base2_exponent as u32)
-				.ok_or_else(|| crate::DeserializeErrorKind::Invalid(format!("Expected integer, got {:?}", source)))?;
+				.ok_or_else(|| crate::DeserializeErrorKind::IntFraction(source.into()))?;
 		} else if self.base2_exponent > 0 {
 			n = n.checked_shl(self.base2_exponent as u32)
-				.ok_or_else(|| crate::DeserializeErrorKind::Invalid(format!("{} is too big", source)))?;
+				.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(source.into()))?;
 		}
 
 		if self.base10_exponent < 0 {
 			let pow = 10u128.checked_pow((-self.base10_exponent) as u32)
-				.ok_or_else(|| crate::DeserializeErrorKind::Invalid(format!("{} is too big", source)))?;
+				.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(source.into()))?;
 			if n % pow != 0 {
-				return Err(crate::DeserializeErrorKind::Invalid(format!("Expected integer, got {:?}", source)));
+				return Err(crate::DeserializeErrorKind::IntFraction(source.into()));
 			}
 			n /= pow;
 		} else if self.base10_exponent > 0 {
 			let pow = 10u128.checked_pow(self.base10_exponent as u32)
-				.ok_or_else(|| crate::DeserializeErrorKind::Invalid(format!("{} is too big", source)))?;
+				.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(source.into()))?;
 			n = n.checked_mul(pow)
-				.ok_or_else(|| crate::DeserializeErrorKind::Invalid(format!("{} is too big", source)))?;
+				.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(source.into()))?;
 		}
 
 		Ok(n)
 	}
 
 	pub fn to_sint<
-		T: std::convert::TryFrom<u128> + std::ops::Neg
+		T: std::convert::TryFrom<i128> + std::ops::Neg
 	>(
 		&self,
 		source: &str,
 	) -> Result<T, crate::DeserializeErrorKind>
 	where
-		<T as std::convert::TryFrom<u128>>::Error: std::fmt::Debug,
+		<T as std::convert::TryFrom<i128>>::Error: std::fmt::Debug,
 		<T as std::ops::Neg>::Output: std::convert::Into<T>,
 	{
-		let mut i: T = self.to_uint(source)?;
-		if !self.positive {
-			i = (-i).into();
+		let mut i;
+		if !self.positive && self.numerator == (i128::max_value() as u128) + 1 {
+			i = i128::min_value();
+		} else {
+			i = self.to_uint(source)?;
+			if !self.positive {
+				i = (-i).into();
+			}
 		}
-		Ok(i)
+
+		return T::try_from(i)
+			.map_err(|_| crate::DeserializeErrorKind::IntOverflow(source.into()))
 	}
 
 	pub fn to_uint<T: std::convert::TryFrom<u128>>(&self, source: &str)
@@ -57,9 +64,18 @@ impl Number {
 		<T as std::convert::TryFrom<u128>>::Error: std::fmt::Debug,
 	{
 		T::try_from(self.to_int(source)?)
-			.map_err(|_| crate::DeserializeErrorKind::Invalid(format!("{} is too big", source)))
+			.map_err(|_| crate::DeserializeErrorKind::IntOverflow(source.into()))
 	}
 
+	pub fn to_f64(&self) -> f64 {
+		let mut n = self.numerator as f64;
+		if !self.positive {
+			n = -n;
+		}
+		n *= 2f64.powi(self.base2_exponent.into());
+		n *= 10f64.powi(self.base10_exponent.into());
+		n
+	}
 }
 
 pub(crate) fn parse_num(mut s: &str, location: crate::Location) -> Result<Number, crate::de::DeserializeError> {
@@ -89,29 +105,50 @@ pub(crate) fn parse_num(mut s: &str, location: crate::Location) -> Result<Number
 
 	bytes = &bytes[consume..];
 
-	let (mut n, len) = parse_num_base(base, bytes);
+	let (mut n, len) = parse_num_base(base, bytes)
+		.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(s.into()).at(location))?;
 	bytes = &bytes[len as usize..];
 
-	let mut base2_exponent = 0;
-	let mut base10_exponent = 0;
+	let mut base2_exponent = 0i16;
+	let mut base10_exponent = 0i16;
 
 	if bytes.get(0) == Some(&b'.') {
-		let (d, d_len) = parse_num_base(base, &bytes[1..]);
-		n *= (base as u128).pow(d_len as u32);
-		n += d;
+		let (d, d_len) = parse_num_base(base, &bytes[1..])
+			.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(s.into()).at(location))?;
+		n = (base as u128).checked_pow(d_len as u32)
+			.and_then(|e| e.checked_mul(n))
+			.and_then(|n| n.checked_add(d))
+			.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(s.into()).at(location))?;
+
 		match base {
-			2 => base2_exponent += d_len,
-			8 => base2_exponent += d_len * 3,
-			10 => base10_exponent += d_len,
-			16 => base2_exponent += d_len * 4,
+			2 => {
+				base2_exponent = base2_exponent.checked_sub(d_len)
+					.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(s.into()).at(location))?;
+			}
+			8 => {
+				base2_exponent = d_len.checked_mul(3)
+					.and_then(|e| base2_exponent.checked_sub(e))
+					.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(s.into()).at(location))?;
+			}
+			10 => {
+				base10_exponent = base10_exponent.checked_sub(d_len)
+					.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(s.into()).at(location))?;
+			}
+			16 => {
+				base2_exponent = d_len.checked_mul(4)
+					.and_then(|e| base2_exponent.checked_sub(e))
+					.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(s.into()).at(location))?;
+			}
 			_ => unreachable!(),
 		}
-		bytes = &bytes[d_len as usize..];
+		bytes = &bytes[1 + d_len as usize..];
 	}
 
 	let (b2, b10) = parse_num_exponent(base, bytes, location)?;
-	base2_exponent += b2;
-	base10_exponent += b10;
+	base2_exponent = base2_exponent.checked_add(b2)
+		.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(s.into()).at(location))?;
+	base10_exponent = base10_exponent.checked_add(b10)
+		.ok_or_else(|| crate::DeserializeErrorKind::IntOverflow(s.into()).at(location))?;
 
 	Ok(Number {
 		positive,
@@ -121,8 +158,8 @@ pub(crate) fn parse_num(mut s: &str, location: crate::Location) -> Result<Number
 	})
 }
 
-fn parse_num_base(base: u8, mut bytes: &[u8]) -> (u128, i16) {
-	let mut n = 0;
+fn parse_num_base(base: u8, bytes: &[u8]) -> Option<(u128, i16)> {
+	let mut n = 0u128;
 	let mut len = 0i16;
 
 	while let Some(c) = bytes.get(len as usize) {
@@ -132,8 +169,8 @@ fn parse_num_base(base: u8, mut bytes: &[u8]) -> (u128, i16) {
 			b'a'..=b'f' |
 			b'A'..=b'F' => {
 				if let Some(d) = (*c as char).to_digit(base.into()) {
-					n *= base as u128;
-					n += d as u128;
+					n = n.checked_mul(base.into())?
+						.checked_add(d.into())?;
 				} else {
 					break
 				}
@@ -143,7 +180,7 @@ fn parse_num_base(base: u8, mut bytes: &[u8]) -> (u128, i16) {
 		len += 1;
 	}
 
-	(n, len)
+	Some((n, len))
 }
 
 fn parse_num_exponent(base: u8, bytes: &[u8], mut location: crate::Location) -> Result<(i16, i16), crate::de::DeserializeError> {
@@ -155,9 +192,7 @@ fn parse_num_exponent(base: u8, bytes: &[u8], mut location: crate::Location) ->
 			1 if bytes[1] == b'i' => Ok((bin, 0)),
 			_ => {
 				location.add_columns(2);
-				Err(crate::de::DeserializeErrorKind::Invalid(
-					format!("Unexpected characters after number {:?}", s))
-					.at(location))
+				Err(crate::de::DeserializeErrorKind::NumTrailing(s.into()).at(location))
 			}
 		}
 	};
@@ -179,22 +214,16 @@ fn parse_num_exponent(base: u8, bytes: &[u8], mut location: crate::Location) ->
 		Some('a') => bindec(&s[1..], 70, 21),
 		Some('e') => {
 			if base != 10 {
-				return Err(crate::de::DeserializeErrorKind::Invalid(
-					"e exponents are only supported in base 10".into())
-					.at(location))
+				return Err(crate::de::DeserializeErrorKind::NumExpBase.at(location))
 			}
 
 			location.add_columns(1);
 
 			let e = s[1..].parse()
-				.map_err(|e| crate::DeserializeErrorKind::Invalid(
-					format!("Could not parse {:?} as an exponent: {}", &s[1..], e))
-					.at(location))?;
+				.map_err(|e| crate::DeserializeErrorKind::NumExpInvalid(e).at(location))?;
 			Ok((0, e))
 		}
-		Some(_) => Err(crate::de::DeserializeErrorKind::Invalid(
-			format!("Unexpected characters after number {:?}", s))
-			.at(location)),
+		Some(_) => Err(crate::de::DeserializeErrorKind::NumTrailing(s.into()).at(location)),
 		None => Ok((0, 0)),
 	}
 }
